# Copyright 2014 Canonical
# All Rights Reserved

"""Rest Scopes server."""

import importlib
import json
import os

import yaml

# this module is generated by a "make build"
from versioninfo import version_info

REVNO_HEADER = ('X-Bzr-Revision-Number', str(version_info['revno']))


# load config
with open(os.environ['REST_SCOPES_CONFIG']) as fh:
    config = yaml.load(fh)


class App(object):
    """The WSGI application."""

    def __init__(self):
        # load scope info and modules
        scope_info = []
        self.scope_apps = {}
        scopes_config_file = os.path.join(os.environ['ROOTDIR'],
                                          config['scopes_config'])
        with open(scopes_config_file) as fh:
            info = yaml.load(fh)
        for scope_id, data in info.items():
            base_url = data['base_url']
            if base_url[0] == '/':
                # internal path
                base_url = config['server'] + base_url

            scope_info.append({
                'name': data['name'],
                'description': data['description'],
                'base_url': base_url,
                'invisible': data['invisible'],
            })
            mod_path = "%s.%s" % (config['scopes_package'], scope_id)
            mod = importlib.import_module(mod_path)
            self.scope_apps[scope_id] = mod.App()
        self.scope_list = json.dumps(scope_info)

    def _generic_response(self, status, environ, start_response,
                          message="", **extra_headers):
        response_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(message))),
            REVNO_HEADER,
        ]
        response_headers.extend(extra_headers.items())
        start_response(status, response_headers)
        return [message]

    def _error(self, status, environ, start_response, message="",
               **extra_headers):
        data = {"error": status}
        if message:
            data['message'] = message
        return self._generic_response(status, environ, start_response,
                                      json.dumps(data, indent=4),
                                      **extra_headers)

    def __call__(self, environ, start_response):
        """Serve the request."""
        path = environ["PATH_INFO"].rstrip("/")
        if path == "/remote-scopes":
            return self.list_remote_scopes(environ, start_response)

        # may be a scope!
        scope_name = path.split("/")[1]
        try:
            scope = self.scope_apps[scope_name]
        except KeyError:
            return self._error("404 NOT FOUND", environ, start_response)

        scope_path = path[len(scope_name) + 1:]  # +1 because of initial /
        if len(scope_path) < 2:
            msg = "Malformed URL: need an action after the scope name"
            return self._error("400 BAD REQUEST", environ, start_response, msg)

        # fix environment and serve
        environ["PATH_INFO"] = scope_path
        return scope(environ, start_response)

    def list_remote_scopes(self, environ, start_response):
        """List all the configured scopes."""
        return self._generic_response("200 OK", environ, start_response,
                                      self.scope_list)
