# Copyright 2014 Canonical
# All Rights Reserved

"""A base Simple Application to provide common functionality to scopes.

When building a new builtin scope, just create a Python class (inheriting
this SimpleApp) and put it in a file in the scopes directory. Check the
src/scopes/wikipedia.py scope as a fully working example and the SimpleApp
docstring for more info.

Of course, if you want more complex needs and want to write the whole
stack, you can do it.
"""

import urlparse
import json

# the locale can be formed only by these chars
ALLOWED_LOCALE_CHARS = set(
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_")

# some defaults for the requests
DEFAULT_LIMIT = 10
DEFAULT_COUNTRY = 'US'
DEFAULT_LOCALE = "en"

# the categories to use, with some renderers
CATEGORY_DEFAULT = 'cat1'
RENDERER_DEFAULT = {
    'template': {'renderer': 'grid', 'overlay-mode': None},
    'components': {'title':'title','art':'art'},
}

# to separate the result chunks
CHUNK_SEPARATOR = '\r\n'


class SimpleApp(object):
    """A simple wsgi app that serves the two mandatory endpoints.

    Subclass at will, feel free to not use it at all if have more
    complex needs.

    Then overwrite the 'search' and 'preview' methods. These will be
    automatically called, on requests. Both methods will receive the request
    parameters, and need to 'yield' the categories/results/etc. The
    specification of what these methods will receive and what they
    must yield is in FIXME.
    """

    categories = {
        CATEGORY_DEFAULT: {
            'id': CATEGORY_DEFAULT,
            'title': 'Category Default',
            'render_template': json.dumps(RENDERER_DEFAULT),
            'icon': "FIXME",
        },
    }

    def error(self, status, environ, start_response, error_msg=None):
        """Answer an error to the client."""
        data = {"error": status}
        if error_msg is not None:
            data['error_msg'] = error_msg
        message = json.dumps(data, indent=4)

        response_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(message))),
        ]
        start_response(status, response_headers)
        return [message]

    def _get_search_args(self, environ):
        """Get the parameters for the 'search', checking all is ok."""
        # get the query string, decode, build the query dictionary
        qs = environ.get("QUERY_STRING", "").decode("utf8")
        qd = dict(urlparse.parse_qsl(qs))

        # latitude and longitude must be paired
        lat = float(qd['latitude']) if 'latitude' in qd else None
        lng = float(qd['longitude']) if 'longitude' in qd else None
        if lat is None and lng is not None or lng is None and lat is not None:
            msg = "latitude and longitude must be paired, both present or none"
            raise ValueError(msg)

        # used to get bad locales
        locale = qd.get('locale')
        if locale is None:
            locale = DEFAULT_LOCALE
        else:
            if set(locale) - ALLOWED_LOCALE_CHARS:
                raise ValueError("Bad locale")
            if locale == 'C':
                locale = 'en_US'

        # get all the rest of parameters, checking also type
        args = {
            'query': qd['q'],
            'session_id': qd.get('session_id'),
            'query_id': int(qd['query_id']) if 'query_id' in qd else None,
            'locale': locale,
            'platform': qd['platform'],
            'country': qd.get('country', DEFAULT_COUNTRY),
            'latitude': lat,
            'longitude': lng,
            'limit': int(qd['limit'], DEFAULT_LIMIT),
        }
        return args

    def _get_preview_args(self, environ):
        """Get the parameters for the 'preview', checking all is ok."""
        # FIXME: do this
        print "============ check preview"

    def search(self, **kwargs):
        """To be overwritten by subclass."""
        raise NotImplementedError("Need to overwrite 'search' method.")

    def preview(self, **kwargs):
        """To be overwritten by subclass."""
        raise NotImplementedError("Need to overwrite 'preview' method.")

    def __call__(self, environ, start_response):
        """Serve the request."""
        path = environ["PATH_INFO"]

        # select the methods to serve the request
        if path == '/search':
            args_checker = self._get_search_args
            serving_method = self.search
        elif path == '/preview':
            args_checker = self._get_preview_args
            serving_method = self.preview
        else:
            return self.error("404 NOT FOUND", environ, start_response)

        # get parameters
        try:
            args = args_checker(environ)
        except ValueError as err:
            msg = "Problem in parameters: " + str(err)
            return self.error("400 BAD REQUEST", environ, start_response, msg)
        except KeyError as err:
            msg = "Missing parameter " + str(err)
            return self.error("400 BAD REQUEST", environ, start_response, msg)

        def chunk_generator():
            """Send encoded chunks to the user."""
            for result in serving_method(**args):
                msg = json.dumps(result)
                yield msg + CHUNK_SEPARATOR

        # start the response and send the chunks to the user
        response_headers = [
#            ('Content-type', 'text/vnd.canonical.json-stream'),
        ]
        start_response("200 OK", response_headers)
        return chunk_generator()
